"""
histhelper is a collection of helper functions to make histograms.
Developed as part of Matthew Feickert's ATLAS Authorship Service Task
# Author: Matthew Feickert <feickert@cern.ch>
# Date: 2017-03-17
"""
import ROOT
# from rootpy.plotting import Hist


def get_max_value(h):
    """
    Get the height of the histogram

    Args:
        h (TH1): The histogram

    Returns:
        (int, float) The height of the histogram (the height of the bin with the
                     maximum content)
    """
    return h.GetBinContent(h.GetMaximumBin())


def get_max_value_error(h):
    """
    Get the error of the maximum bin of a histogram

    Args:
        h (TH1): The histogram

    Returns:
        (int, float) The error of the of the bin with the
                     maximum content of the histogram
    """
    return h.GetBinError(h.GetMaximumBin())


def get_hist_height(h1, h2):
    # TODO: Make a better version of this for scaling
    """
    Find the maximum height between two histograms and return a scaled height

    Args:
        h1 (TH1): One of the histograms considered
        h2 (TH1): One of the histograms considered

    Returns:
        (int, float) The height of the tallest histogram scaled relative to the
                     difference in height between the histograms
    """
    max_value = (get_max_value(h1) + get_max_value_error(h1)) if (
        get_max_value(h1) > get_max_value(h2)) else (get_max_value(h2) + get_max_value_error(h2))
    next_value = get_max_value(h2) if (
        get_max_value(h1) > get_max_value(h2)) else get_max_value(h1)

    if (next_value < max_value / 2):
        return max_value + next_value / 3
    # return (3 * max_value - next_value) / 2
    if ((max_value - next_value) > 100):
        return max_value + (max_value - next_value)
    else:
        return max_value + (max_value - next_value) * (0.25)


def get_bin_width_units(h):
    """
    Get the width of the bins in a histogram with fixed bin width in the units of the
    x-axis (precision set to up to 3 decimal places)

    Args:
        h (TH1): The fixed bin width histogram

    Returns:
        (int, float) The width of a bin given in the units of the histogram's x-axis
    """
    number_bins = h.FindLastBinAbove(-1)
    hist_length = h.GetXaxis().GetBinUpEdge(number_bins) - \
        h.GetXaxis().GetBinLowEdge(h.FindFirstBinAbove(-1))
    bin_width_units = float(hist_length / number_bins)
    if ((bin_width_units).is_integer()):
        bin_width_units = int(bin_width_units)
    # allow only up to 3 digits after the decimal point
    if (int(str(bin_width_units)[::-1].find('.')) > 3):
        bin_width_units = round(float("{0:.4f}".format(bin_width_units)), 3)
    return bin_width_units


def get_color_wheel_number(color):
    """
    Get the color wheel number of the color passed. Default color is black.
    * https://root.cern.ch/doc/master/classTColor.html
    * https://root.cern.ch/doc/master/Rtypes_8h.html

    Args:
        color (str): The color requested

    Returns:
        The integer that corresponds to the color requested
    """
    color_wheel = {
        'white': 0,
        'black': 1,
        'gray': 920,
        'grey': 920,
        'red': 632,
        'green': 416,
        'blue': 600,
        'yellow': 400,
        'magenta': 616,
        'cyan': 432,
        'orange': 800,
        'spring': 820,
        'teal': 840,
        'azure': 860,
        'violet': 880,
        'pink': 900
    }
    color = str(color).lower()
    if color in color_wheel:
        return color_wheel[color]
    else:
        print("ERROR: The color {} is not defined. Defaulting to black.".format(color))
        return color_wheel['black']


def flag_ratio(g, threshold):
    """
    Flag a ratio plot if the ratio exceeds a set threshold

    Args:
        g (TGraph): The TGraph from the TRatioPlot
        threshold (int, double): threshold value not to exceed in ratio

    Returns:
        bool
    """
    for itr in list(range(0, g.GetN())):
        if g.GetY()[itr] > threshold:
            print("y-value at is {}".format(g.GetY()[itr]))
            return True


def draw_ratioplot(h1, h2=None, fit_option=None, has_log_scale_y=False,
                   threshold=None, draw_options=None,
                   h1_line_color="black", h2_line_color="blue",
                   legend_labels=["dataset 2", "dataset 1"],
                   save_dir=None, save_name=None, write_file=None):
    """
    Draws and saves a TRatioPlot. A TCanvas object must exist in scope when the
    TRatioPlot constructor is called or a seg fault will occur.

    Args:
        h1 (TH1): A TH1 to be drawn and saved. If there is no h1 it must have a fit
                    associated with it.
        h2 (TH1): A TH1 to be drawn in ratio to h1 and saved
        fit_options (str): TRatioPlot calculation and residual options (divsym, diff, diffsig)
        has_log_scale_y (bool): Determines if y-axis plotted on linear or log scale
        threshold (int, double): threshold value that if exceeded flags the ratio plot
        draw_options (str): TH1 draw options
        h1_line_color (str): The line color of h1
        h2_line_color (str): The line color of h2
        legend_labels ([str, str]): The lables displayed in the legend
        save_dir (str): The path to the directory where the canvas will be saved
        save_name (str): The name of the saved canvas
        write_file (TFile): The TFile that the TH1 will be written to

    Returns:
        None
    """
    can = ROOT.TCanvas("c", "c", 700, 500)
    can.cd()
    h1 = h1.Clone()
    if h2:
        h2 = h2.Clone()
        if fit_option:
            ratio_plot = ROOT.TRatioPlot(h1, h2, fit_option)
        else:
            ratio_plot = ROOT.TRatioPlot(h1, h2)
        if h1_line_color:
            h1.SetLineColor(get_color_wheel_number(h1_line_color))
        if h2_line_color:
            h2.SetLineColor(get_color_wheel_number(h2_line_color))

        h1.SetMarkerStyle(20)
        h1.SetMarkerSize(0.5)
        # h2.SetFillColorAlpha(get_color_wheel_number(h2_line_color), 0.2)
        h2.SetMarkerStyle(22)
        h2.SetMarkerSize(0.5)
        h2.SetMarkerColor(get_color_wheel_number(h2_line_color))
        h2.SetFillColorAlpha(get_color_wheel_number(h2_line_color), 0.6)
        h2.SetFillStyle(3344)
    else:
        if fit_option:
            ratio_plot = ROOT.TRatioPlot(h1, fit_option)
        else:
            ratio_plot = ROOT.TRatioPlot(h1)

    ratio_plot.SetSeparationMargin(0.01)
    ratio_plot.SetH1DrawOpt("E1")
    ratio_plot.SetH2DrawOpt("E2")
    if draw_options:
        ratio_plot.Draw(draw_options)
    else:
        ratio_plot.Draw()

    ratio_plot.GetLowerRefGraph().SetLineColor(get_color_wheel_number("black"))
    # Ensure all points are visible
    if (has_log_scale_y):
        ratio_plot.GetUpperPad().SetLogy()
    else:
        h1.SetMaximum(get_hist_height(h1, h2))
    # Enforce reasonable boundaries on the ratio plot
    if (fit_option == None or fit_option == "divsym"):
        ratio_plot.GetLowerRefGraph().SetMinimum(-0.01)
        if ratio_plot.GetLowerRefGraph().GetHistogram().GetMaximum() > 2.0:
            ratio_plot.GetLowerRefGraph().SetMaximum(2.05)
        elif ratio_plot.GetLowerRefGraph().GetHistogram().GetMaximum() < 1.2:
            ratio_plot.GetLowerRefGraph().SetMaximum(1.2)

    if len(legend_labels) > 1:
        if (fit_option == "diff"):
            ratio_plot.GetLowerRefYaxis().SetTitle(
                "(" + legend_labels[0] + " - " + legend_labels[1] + ")")
        if (fit_option == "diffsig"):
            ratio_plot.GetLowerRefYaxis().SetTitle(
                "(" + legend_labels[0] + " - " + legend_labels[1] + ") / #sigma")
        elif (fit_option == None or fit_option == "divsym"):
            ratio_plot.GetLowerRefYaxis().SetTitle(
                legend_labels[0] + " / " + legend_labels[1])
    else:
        if (fit_option == "diff"):
            ratio_plot.GetLowerRefYaxis().SetTitle(
                legend_labels[0] + " - dataset 1")
        if (fit_option == "diffsig"):
            ratio_plot.GetLowerRefYaxis().SetTitle(
                "(" + legend_labels[0] + " - dataset 1) / #sigma")
        elif (fit_option == None or fit_option == "divsym"):
            ratio_plot.GetLowerRefYaxis().SetTitle(
                legend_labels[0] + " / dataset 1")

    ratio_plot.GetUpperPad().cd()
    legend = ROOT.TLegend(0.75, 0.7, 0.9, 0.9)
    legend.SetBorderSize(0)
    legend.SetFillStyle(0)  # make legend transparent
    is_flagged = False
    if (fit_option == None or fit_option == "divsym"):
        if threshold:
            is_flagged = flag_ratio(ratio_plot.GetLowerRefGraph(), threshold)
            if (is_flagged):
                legend.AddEntry(0, "FLAGGED", "").SetTextColor(
                    get_color_wheel_number("red"))
    legend.AddEntry(h1, legend_labels[0], "LEP")
    if len(legend_labels) > 1:
        legend.AddEntry(h2, legend_labels[1], "FEP")
    else:
        legend.AddEntry(h2, "dataset 1", "FEP")
    legend.Draw()

    if save_dir:
        can.Update()
        if save_name:
            can.SaveAs(save_dir + "/" + save_name + ".pdf")
            can.SaveAs(save_dir + "/" + save_name + ".eps")
            can.SaveAs(save_dir + "/" + save_name + ".png")
            # can.SaveAs(save_dir + "/" + save_name + ".jpg")
            if is_flagged:
                can.SaveAs(save_dir + "/flagged_" + save_name + ".pdf")
                can.SaveAs(save_dir + "/flagged_" + save_name + ".eps")
                can.SaveAs(save_dir + "/flagged_" + save_name + ".png")
                # can.SaveAs(save_dir + "/flagged_" + save_name + ".jpg")
        elif save_name == None:
            can.SaveAs(save_dir + "/" + "ratio.pdf")
    if write_file:
        write_file.cd()
        ratio_plot.Write()
    # cleanup canvas
    # TODO: Have this patch added to ROOT v6.08
    # https://root-forum.cern.ch/t/tratioplot-seg-fault-in-pyroot/22297
    # can.Destructor()

if __name__ == "__main__":
    import os
    ROOT.gROOT.SetBatch(True)

    # Modification of $ROOTSYS/tutorials/hist/ratioplot1.C
    h1 = ROOT.TH1D("h1", "", 50, 0, 10)
    h2 = ROOT.TH1D("h2", "", 50, 0, 10)
    f1 = ROOT.TF1("f1", "exp(- x/[0] )")
    h1.SetLineColor(get_color_wheel_number("black"))
    h2.SetLineColor(get_color_wheel_number("blue"))
    h1.SetStats(False)
    h2.SetStats(False)
    f1.SetParameter(0, 3)
    h1.FillRandom("f1", 1900)
    h2.FillRandom("f1", 2500)
    h1.Sumw2()
    # h2.Scale(1.9 / 2.)
    h1.GetXaxis().SetTitle("x")
    h1.GetYaxis().SetTitle("y")

    # ratio > 1
    draw_ratioplot(h1, h2=h2, has_log_scale_y=True,
                   threshold=5, legend_labels=["test", "nominal"],
                   save_dir=os.getcwd() + "/output", save_name="ratio")
