import ROOT
from histhelper import draw_ratioplot
import os


def get_hist(file, hist_name, dir_name=None):
    """
    Retrieves TH1 from TFile

    Args:
        file (TFile): The file in which the histograms are located
        dir_name (str): The TDirectory in which the histogram is located
        hist_name (str): The name of the TH1

    Returns:
        The TH1 requested
    """
    if dir_name:
        dir = file.GetDirectory(dir_name)
        hist = dir.Get(hist_name)
    else:
        hist = file.Get(hist_name)

    return hist


def get_bin_width_units(h):
    """
    Get the width of the bins in a histogram with fixed bin width in the units of the
    x-axis (precision set to up to 3 decimal places)

    Args:
        h (TH1): The fixed bin width histogram

    Returns:
        (int, float) The width of a bin given in the units of the histogram's x-axis
    """
    number_bins = h.FindLastBinAbove(-1)
    hist_length = h.GetXaxis().GetBinUpEdge(number_bins) - \
        h.GetXaxis().GetBinLowEdge(h.FindFirstBinAbove(-1))
    bin_width_units = float(hist_length / number_bins)
    if ((bin_width_units).is_integer()):
        bin_width_units = int(bin_width_units)
    # allow only up to 3 digits after the decimal point
    if (int(str(bin_width_units)[::-1].find('.')) > 3):
        bin_width_units = round(float("{0:.4f}".format(bin_width_units)), 3)
    return bin_width_units


def label_hist(h, title=None, x_axis_title=None, y_axis_title=None,
               y_scale_min=None, y_scale_max=None):
    """
    Apply titles, axis labes, and y-axis ranges to the TH1 passed

    Args:
        h (TH1): The TH1 to have labels applied
        title (str): The title of the histogram
        x_axis_title (str): The label for the histogram's x-axis
        y_axis_title (str): The label for the histogram's y-axis
        y_scale_min (int, float): The minimum value of the y-axis
        y_scale_max (int, float): The maximum value of the y-axis

    Returns:
        None
    """
    h.Sumw2()
    h.SetTitle(title)
    if x_axis_title:
        h.GetXaxis().SetTitle(x_axis_title)
    if y_axis_title:
        h.GetYaxis().SetTitle(y_axis_title)
    else:
        h.GetYaxis().SetTitle("Count / " + str(get_bin_width_units(h)) + " Units")
    h.SetStats(0)
    if y_scale_min:
        h.SetMinimum(y_scale_min)
    if y_scale_max:
        h.SetMaximum(y_scale_max)


def draw_hist(hist, has_log_scale_y=False, has_log_scale_x=False, draw_options=None,
              x_range_low=None, x_range_high=None,
              y_range_low=None, y_range_high=None,
              save_dir=None, save_name=None, write_file=None):
    """
    Draws and saves a TH1

    Args:
        hist (TH1): The TH1 to be drawn and saved
        has_log_scale_y (bool): Determines if y-axis plotted on linear or log scale
        has_log_scale_x (bool): Determines if x-axis plotted on linear or log scale
        draw_options (str): TH1 draw options
        x_range_low (int, float): The left extreme range of the plot x-axis
        x_range_high (int, float): The right extreme range of the plot x-axis
        y_range_low (int, float): The bottom extreme range of the plot y-axis
        y_range_high (int, float): The top extreme range of the plot y-axis
        save_dir (str): The path to the directory where the canvas will be saved
        save_name (str): The name of the saved canvas
        write_file (TFile): The TFile that the TH1 will be written to

    Returns:
        None
    """
    can = ROOT.TCanvas("c", "c", 700, 500)
    can.cd()
    if (has_log_scale_y):
        can.SetLogy()
    if (has_log_scale_x):
        can.SetLogx()
    if x_range_low and x_range_high:
        hist.GetXaxis().SetRangeUser(x_range_low, x_range_high)
    if y_range_low and y_range_high:
        hist.GetYaxis().SetRangeUser(y_range_low, y_range_high)
    if draw_options:
        hist.Draw(draw_options)
    else:
        hist.Draw()
    if save_dir:
        can.Update()
        if save_name:
            can.SaveAs(save_dir + "/" + save_name + ".pdf")
            can.SaveAs(save_dir + "/" + save_name + ".eps")
            can.SaveAs(save_dir + "/" + save_name + ".png")
            can.SaveAs(save_dir + "/" + save_name + ".jpg")
        elif save_name == None:
            can.SaveAs(save_dir + "/" + "hist.pdf")
    if write_file:
        write_file.cd()
        hist.Write()
    # cleanup canvas
    can.Destructor()

if __name__ == "__main__":
    import os
    import sys
    # sys.argv.append('-b')
    # ROOT.gROOT.SetBatch(True)

    make_bool = False
    if make_bool:
        output_file = ROOT.TFile(os.getcwd() + "/HistOut.root", "RECREATE")
        # Modification of $ROOTSYS/tutorials/hist/ratioplot1.C
        h1 = ROOT.TH1D("h1", "", 50, 0, 10)
        h2 = ROOT.TH1D("h2", "", 50, 0, 10)
        f1 = ROOT.TF1("f1", "exp(- x/[0] )")
        h1.SetLineColor(1)  # black
        h2.SetLineColor(600)  # blue
        h1.SetStats(False)
        h2.SetStats(False)
        f1.SetParameter(0, 3)
        h1.FillRandom("f1", 1900)
        h2.FillRandom("f1", 2500)
        h1.Sumw2()
        h1.GetXaxis().SetTitle("x")
        h1.GetYaxis().SetTitle("y")
    else:
        in_file = ROOT.TFile(os.getcwd() + "/HistOut.root", "READ")
        output_file = ROOT.TFile(os.getcwd() + "/PlotOut.root", "RECREATE")

        h1 = get_hist(in_file, "h1")
        label_hist(h1, "", "", "")
        h2 = get_hist(in_file, "h2")
        label_hist(h2, "", "", "")

    draw_hist(hist=h1, save_dir=os.getcwd(),
              save_name="h1", write_file=output_file)
    draw_hist(hist=h1, has_log_scale_y=True, save_dir=os.getcwd(),
              save_name="h1_log", write_file=output_file)

    draw_hist(hist=h2, save_dir=os.getcwd(),
              save_name="h2", write_file=output_file)
    draw_hist(hist=h2, has_log_scale_y=True, save_dir=os.getcwd(),
              save_name="h2_log", write_file=output_file)

    draw_ratioplot(h1, h2=h2, has_log_scale_y=True,
                   threshold=5, legend_labels=["test", "nominal"],
                   save_dir=os.getcwd(), save_name="ratio")
